from uuid import uuid4

from .property import PropertyDef


class CategorizationDef:
    def __init__(self, properties: list):
        self.properties = properties

    @classmethod
    def get_default(cls):
        return cls(properties=[])

    def add_multiple_choice_category(
        self,
        name: str,
        options: list,
        id: str = None,
        is_required: bool = False,
        description: str = "",
        render_value: bool = False,
        default_value: bool = "",
        is_per_frame: bool = False,
    ):
        for category in self.properties:
            if category["name"] == name:
                return print(f"[ERROR] {name} already exists")
        self.properties.append(
            PropertyDef.multiple_choice_property(
                name=name,
                options=options,
                id=id,
                default_value=default_value,
                description=description,
                render_value=render_value,
                is_required=is_required,
                is_per_frame=is_per_frame,
            )
        )

    def add_multiple_selection_category(
        self,
        name: str,
        options: list,
        id: str = None,
        is_required: bool = False,
        description: str = "",
        render_value: bool = False,
        default_value: list = [],
        is_per_frame: bool = False,
    ):
        for category in self.properties:
            if category["name"] == name:
                return print(f"[ERROR] {name} already exists")
        self.properties.append(
            PropertyDef.multiple_selection_property(
                name=name,
                options=options,
                id=id,
                default_value=default_value,
                description=description,
                render_value=render_value,
                is_required=is_required,
                is_per_frame=is_per_frame,
            )
        )

    def add_free_response_category(
        self,
        name: str,
        id: str = None,
        blank: bool = True,
        constraints: dict = {
            "digit": True,
            "space": True,
            "special": True,
            "alphabet": True,
        },
        description: str = "",
        render_value: bool = False,
        default_value: str = "",
        is_per_frame: bool = False,
    ):
        for category in self.properties:
            if category["name"] == name:
                return print(f"[ERROR] {name} already exists")
        id = str(uuid4()) if id is None else id
        self.properties.append(
            PropertyDef.free_response_property(
                name=name,
                id=id,
                blank=blank,
                constraints=constraints,
                description=description,
                render_value=render_value,
                default_value=default_value,
                is_per_frame=is_per_frame,
            )
        )

    def remove_category(self, name: str):
        for category in self.properties:
            if category["name"] == name:
                self.properties.remove(category)
                return

    def remove_category_by_id(self, id: str):
        for category in self.result["properties"]:
            if category["id"] == id:
                self.properties.remove(category)
                return

    def to_dict(self):
        return {
            "properties": self.properties,
        }

    @classmethod
    def from_dict(cls, categorization_dict: dict):
        return cls(properties=categorization_dict["properties"])



from abc import ABC, abstractmethod
from .utils import unique_string_builder


class LabelCreator(ABC):
    def __init__(self, object_classes_map):
        self.object_classes_map = object_classes_map

    @abstractmethod
    def create(self):
        pass

    @abstractmethod
    def from_dict(self):
        pass

    def _get_meta(self, class_name, class_type):
        return {
            "z_index": 1,
            "visible": True,
            "alpha": 1,
            "color": self.object_classes_map[unique_string_builder(
                class_name, class_type
            )]["color"],
        }



from typing import Optional

from ..common.object_interface import ObjectInterfaceDef


class ObjectTrackingDef(ObjectInterfaceDef):
    def __init__(
        self,
        object_groups: list = [],
        object_classes: list = [],
        annotation_types: list = [],
    ):
        super().__init__(
            keypoints=None,
            object_groups=object_groups,
            object_classes=object_classes,
            annotation_types=annotation_types,
        )

    @classmethod
    def get_default(cls):
        return cls(
            object_groups=[],
            object_classes=[],
            annotation_types=[],
        )

    @classmethod
    def from_dict(cls, object_detection_dict: dict):
        return cls(
            object_groups=object_detection_dict["object_groups"],
            object_classes=object_detection_dict["object_classes"],
            annotation_types=object_detection_dict["annotation_types"],
        )

    def add_cuboid(
        self,
        name: str,
        id: Optional[str] = None,
        color: Optional[str] = None,
        properties: list = [],
    ):
        self.add_object_class(name, "cuboid", color, properties, id)

    def add_box(self): raise NotImplementedError

    def add_rbox(self): raise NotImplementedError

    def add_polyline(self): raise NotImplementedError

    def add_polygon(self): raise NotImplementedError

    def add_2dcuboid(self): raise NotImplementedError

    def add_keypoint(self): raise NotImplementedError

    def to_dict(self):
        return {
            "object_groups": self.object_groups,
            "object_classes": self.object_classes,
            "annotation_types": self.annotation_types,
        }



# from phy_credit.common.categorization import CategorizationDef
from ..common import is_camel_version
from .categorization import CategorizationDef
from .label_info import LabelInfo
from .label_interface import LabelInterface
from .legacy.label_info import LabelInfo as LegacyLabelInfo
from .object_detection import ObjectDetectionDef
from .project_info import ProjectInfo
from .property import PropertyDef, PropertyOptionsDef, PropertyOptionsItemDef


def build_label_info(label_interface, result=None):
    if type(label_interface) is dict:
        label_interface = LabelInterface.from_dict(label_interface)

    if is_camel_version(label_interface):
        return LegacyLabelInfo(label_interface, result)
    return LabelInfo(label_interface, result)


# def build_project_info(name: str = None, project_info: dict = None) -> ProjectInfo:
#     if name or project_info:
#         return ProjectInfo(name=name, project_info=project_info)
#     else:
#         return ProjectInfo()


# def build_label_interface(label_interface: dict = None) -> LabelInterface:
#     if label_interface:
#         return LabelInterface(label_interface=label_interface)
#     else:
#         return LabelInterface()


# def build_categorization(categorization: dict = None) -> Categorization:
#     if categorization:
#         return Categorization(categorization=categorization)
#     else:
#         return Categorization()


# def build_category_options(options: list = None) -> CategoryOptions:
#     if options:
#         return CategoryOptions(options=options)
#     else:
#         return CategoryOptions()


# def build_class_property(name: str = None, options: list = None) -> ClassProperty:
#     if name:
#         return ClassProperty(name=name, options=options)
#     else:
#         return ClassProperty()


# def build_object_dection(object: dict = None) -> ObjectDetection:
#     if object:
#         return ObjectDetection(object=object)
#     else:
#         return ObjectDetection()


__all__ = (
    "LabelInfo",
    "LegacyLabelInfo",
    "ObjectDetectionDef",
    "build_label_info",
    "LabelInterface",
    "ProjectInfo",
    "CategorizationDef",
    "PropertyDef",
    "PropertyOptionsDef",
    "PropertyOptionsItemDef",
    # "build_project_info",
    # "build_label_interface",
    # "build_categorization",
    # "build_category_options",
    # "build_class_property",
    # "build_object_dection",
)



import pytest


@pytest.fixture
def mock_image_objects_with_properties():
    return [
        {
            "id": "aaa",
            "class_id": "car-id",
            "class_name": "Car",
            "annotation_type": "box",
            "annotation": {},
            "properties": [
                {
                    "property_id": "Car-color-id",
                    "property_name": "Color",
                    "option_id": "Silver-id",
                    "option_name": "Silver",
                    "type": "radio",
                },
                {
                    "property_id": "Car-color-id",
                    "property_name": "Color",
                    "option_id": "Silver-id",
                    "option_name": "Silver",
                    "type": "radio",
                },
                {
                    "property_id": "Car-color-id",
                    "property_name": "Color",
                    "option_id": "Navy-id",
                    "option_name": "Navy",
                    "type": "radio",
                },
                {
                    "property_id": "Car-color-id",
                    "property_name": "Color",
                    "option_id": "White-id",
                    "option_name": "White",
                    "type": "radio",
                },
                {
                    "property_id": "Car-description-id",
                    "property_name": "Car Description",
                    "value": "This is a sports car",
                    "type": "free response",
                },
            ],
        },
        {
            "id": "bbb",
            "class_id": "car-id",
            "class_name": "Car",
            "annotation_type": "box",
            "annotation": {},
            "properties": [
                {
                    "property_id": "Car-color-id",
                    "property_name": "Color",
                    "option_id": "Silver-id",
                    "option_name": "Silver",
                    "type": "radio",
                },
                {
                    "property_id": "Car-color-id",
                    "property_name": "Color",
                    "option_id": "White-id",
                    "option_name": "White",
                    "type": "radio",
                },
                {
                    "property_id": "Car-description-id",
                    "property_name": "Car Description",
                    "value": "This is a sports car",
                    "type": "free response",
                },
            ],
        },
    ]


@pytest.fixture
def mock_image_classes_properties_count():
    return [
        {
            "class_id": "car-id",
            "property_id": "Car-color-id",
            "property_name": "Color",
            "option_id": "Silver-id",
            "option_name": "Silver",
            "property_option_id": "Car-color-id#Silver-id",
            "count": 3,
        },
        {
            "class_id": "car-id",
            "property_id": "Car-color-id",
            "property_name": "Color",
            "option_id": "Navy-id",
            "option_name": "Navy",
            "property_option_id": "Car-color-id#Navy-id",
            "count": 1,
        },
        {
            "class_id": "car-id",
            "property_id": "Car-color-id",
            "property_name": "Color",
            "option_id": "White-id",
            "option_name": "White",
            "property_option_id": "Car-color-id#White-id",
            "count": 2,
        },
    ]


@pytest.fixture
def mock_preprocessed_properties():
    return [
        {
            "class_id": "person-id",
            "option_id": "female-id",
            "option_name": "Female",
            "property_id": "gender-id",
            "property_name": "Gender",
            "property_option_id": "gender-id#female-id",
        },
        {
            "class_id": "person-id",
            "option_id": "female-id",
            "option_name": "Female",
            "property_id": "gender-id",
            "property_name": "Gender",
            "property_option_id": "gender-id#female-id",
        },
        {
            "class_id": "person-id",
            "option_id": "male-id",
            "option_name": "Male",
            "property_id": "gender-id",
            "property_name": "Gender",
            "property_option_id": "gender-id#male-id",
        },
        {
            "class_id": "person-id",
            "option_id": "true-id",
            "option_name": "True",
            "property_id": "occluded-id",
            "property_name": "Occluded",
            "property_option_id": "occluded-id#true-id",
        },
        {
            "class_id": "person-id",
            "option_id": "false-id",
            "option_name": "False",
            "property_id": "occluded-id",
            "property_name": "Occluded",
            "property_option_id": "occluded-id#false-id",
        },
        {
            "class_id": "person-id",
            "option_id": "false-id",
            "option_name": "False",
            "property_id": "occluded-id",
            "property_name": "Occluded",
            "property_option_id": "occluded-id#false-id",
        },
        {
            "class_id": "person-id",
            "option_id": "false-id",
            "option_name": "False",
            "property_id": "occluded-id",
            "property_name": "Occluded",
            "property_option_id": "occluded-id#false-id",
        },
    ]


@pytest.fixture
def mock_count_properties_result():
    return [
        {
            "class_id": "person-id",
            "option_id": "female-id",
            "option_name": "Female",
            "property_id": "gender-id",
            "property_name": "Gender",
            "property_option_id": "gender-id#female-id",
            "count": 2,
        },
        {
            "class_id": "person-id",
            "option_id": "male-id",
            "option_name": "Male",
            "property_id": "gender-id",
            "property_name": "Gender",
            "property_option_id": "gender-id#male-id",
            "count": 1,
        },
        {
            "class_id": "person-id",
            "option_id": "true-id",
            "option_name": "True",
            "property_id": "occluded-id",
            "property_name": "Occluded",
            "property_option_id": "occluded-id#true-id",
            "count": 1,
        },
        {
            "class_id": "person-id",
            "option_id": "false-id",
            "option_name": "False",
            "property_id": "occluded-id",
            "property_name": "Occluded",
            "property_option_id": "occluded-id#false-id",
            "count": 3,
        },
    ]


@pytest.fixture
def num_properties_flattened():
    return 18


@pytest.fixture
def num_properties_removed_free_response():
    return 10


@pytest.fixture
def num_properties_expanded_options():
    return 14


@pytest.fixture
def num_properties_preprocessed():
    return 14


@pytest.fixture
def mock_video_classes_properties_count():
    return [
        {
            "class_id": "f1e62c91-7241-4d42-8e6c-5a4f79f09b3f",
            "property_id": "ad7647bd-f2a9-4ab4-9ca1-5b7600acfd78",
            "property_name": "Person Subclass",
            "option_id": "627f7de4-5a11-4a98-8618-6ea3439020f3",
            "option_name": "Unknown",
            "property_option_id": "ad7647bd-f2a9-4ab4-9ca1-5b7600acfd78#627f7de4-5a11-4a98-8618-6ea3439020f3",
            "count": 1,
        },
        {
            "class_id": "f1e62c91-7241-4d42-8e6c-5a4f79f09b3f",
            "property_id": "ad7647bd-f2a9-4ab4-9ca1-5b7600acfd78",
            "property_name": "Person Subclass",
            "option_id": "cfd5be2e-695f-468d-8e98-182769728920",
            "option_name": "Client",
            "property_option_id": "ad7647bd-f2a9-4ab4-9ca1-5b7600acfd78#cfd5be2e-695f-468d-8e98-182769728920",
            "count": 1,
        },
        {
            "class_id": "88164746-1f8b-4730-8aec-ebd089cbe9d6",
            "property_id": "ac3bc093-810a-4491-9fc9-3592213aaf3a",
            "property_name": "Mood(PF)",
            "option_id": "1",
            "option_name": "Sad",
            "property_option_id": "ac3bc093-810a-4491-9fc9-3592213aaf3a#1",
            "count": 3,
        },
        {
            "class_id": "88164746-1f8b-4730-8aec-ebd089cbe9d6",
            "property_id": "ac3bc093-810a-4491-9fc9-3592213aaf3a",
            "property_name": "Mood(PF)",
            "option_id": "2",
            "option_name": "Happy",
            "property_option_id": "ac3bc093-810a-4491-9fc9-3592213aaf3a#2",
            "count": 2,
        },
        {
            "class_id": "88164746-1f8b-4730-8aec-ebd089cbe9d6",
            "property_id": "ac3bc093-810a-4491-9fc9-3592213aaf3a",
            "property_name": "Mood(PF)",
            "option_id": "3",
            "option_name": "Thoughtful",
            "property_option_id": "ac3bc093-810a-4491-9fc9-3592213aaf3a#3",
            "count": 2,
        },
        {
            "class_id": "88164746-1f8b-4730-8aec-ebd089cbe9d6",
            "property_id": "fc5cb926-8b6a-443f-8d67-426478f4056f",
            "property_name": "Truncated(PF)",
            "option_id": "1",
            "option_name": "T",
            "property_option_id": "fc5cb926-8b6a-443f-8d67-426478f4056f#1",
            "count": 1,
        },
        {
            "class_id": "88164746-1f8b-4730-8aec-ebd089cbe9d6",
            "property_id": "57d8115c-7a44-45ae-98c6-eb4b2fb9b710",
            "property_name": "Mood",
            "option_id": "3",
            "option_name": "Sad",
            "property_option_id": "57d8115c-7a44-45ae-98c6-eb4b2fb9b710#3",
            "count": 1,
        },
        {
            "class_id": "88164746-1f8b-4730-8aec-ebd089cbe9d6",
            "property_id": "38a1be62-42c2-4c55-b793-136b8c147615",
            "property_name": "Truncated",
            "option_id": "1",
            "option_name": "Yes",
            "property_option_id": "38a1be62-42c2-4c55-b793-136b8c147615#1",
            "count": 1,
        },
        {
            "class_id": "88164746-1f8b-4730-8aec-ebd089cbe9d6",
            "property_id": "df27390a-4f0c-40c7-9f0f-bacf999cdaee",
            "property_name": "Size",
            "option_id": "3",
            "option_name": "S",
            "property_option_id": "df27390a-4f0c-40c7-9f0f-bacf999cdaee#3",
            "count": 1,
        },
        {
            "class_id": "88164746-1f8b-4730-8aec-ebd089cbe9d6",
            "property_id": "c58fe3d6-1693-4995-8e8f-7929b484da1b",
            "property_name": "Occluded",
            "option_id": "1",
            "option_name": "True",
            "property_option_id": "c58fe3d6-1693-4995-8e8f-7929b484da1b#1",
            "count": 1,
        },
    ]


@pytest.fixture
def mock_video_objects():
    return [
        {
            "annotation_type": "keypoint",
            "class_id": "f1e62c91-7241-4d42-8e6c-5a4f79f09b3f",
            "class_name": "Person",
            "frames": [
                {
                    "annotation": {
                        "coord": {
                            "points": [
                                {
                                    "name": "left eye inner corner",
                                    "state": {"valid": True, "visible": True},
                                    "x": 325.8487177544785,
                                    "y": 129.9350831305714,
                                },
                            ]
                        },
                        "meta": {
                            "alpha": 1,
                            "color": "#6648FF",
                            "visible": True,
                            "z_index": 1,
                        },
                    },
                    "num": 0,
                    "properties": [],
                },
                {
                    "annotation": {
                        "coord": {
                            "points": [
                                {
                                    "name": "left eye center",
                                    "state": {"valid": True, "visible": True},
                                    "x": 319.4920928160841,
                                    "y": 126.31307461866575,
                                },
                                {
                                    "name": "left eye inner corner",
                                    "state": {"valid": True, "visible": True},
                                    "x": 325.8487177544785,
                                    "y": 129.9350831305714,
                                },
                                {
                                    "name": "left eye outer corner",
                                    "state": {"valid": True, "visible": True},
                                    "x": 319.15317630532724,
                                    "y": 134.61523555774093,
                                },
                                {
                                    "name": "right eye center",
                                    "state": {"valid": True, "visible": True},
                                    "x": 317.62934558138977,
                                    "y": 155.76259096939575,
                                },
                            ]
                        },
                        "meta": {
                            "alpha": 1,
                            "color": "#6648FF",
                            "visible": True,
                            "z_index": 1,
                        },
                    },
                    "num": 1,
                    "properties": [],
                },
                {
                    "annotation": {
                        "coord": {
                            "points": [
                                {
                                    "name": "nose tip",
                                    "state": {"valid": True, "visible": True},
                                    "x": 328.9325992874725,
                                    "y": 227.90782622868213,
                                },
                            ]
                        },
                        "meta": {
                            "alpha": 1,
                            "color": "#6648FF",
                            "visible": True,
                            "z_index": 1,
                        },
                    },
                    "num": 2,
                    "properties": [],
                },
            ],
            "id": "fee81b6a-8ad2-46c7-819c-89781c730519",
            "properties": [
                {
                    "option_id": "627f7de4-5a11-4a98-8618-6ea3439020f3",
                    "option_name": "Unknown",
                    "property_id": "ad7647bd-f2a9-4ab4-9ca1-5b7600acfd78",
                    "property_name": "Person Subclass",
                    "type": "radio",
                }
            ],
            "tracking_id": 1,
        },
        {
            "annotation_type": "polygon",
            "class_id": "8c0dea92-bbdc-418f-bfa8-86ca249c0b73",
            "class_name": "Grocery Bag",
            "frames": [
                {
                    "annotation": {
                        "coord": {
                            "points": [
                                [
                                    [
                                        {"x": 547.7251977470074,
                                            "y": 16.268694580160915},
                                        {"x": 576.028607118613,
                                            "y": 45.32496429245545},
                                        {"x": 547.7251977470074,
                                            "y": 16.268694580160915},
                                    ]
                                ]
                            ]
                        },
                        "meta": {
                            "alpha": 1,
                            "color": "#FE9573",
                            "visible": True,
                            "z_index": 2,
                        },
                        "multiple": True,
                    },
                    "num": 0,
                    "properties": [],
                },
                {
                    "annotation": {
                        "coord": {
                            "points": [
                                [
                                    [
                                        {"x": 547.7251977470074,
                                            "y": 16.268694580160915},
                                        {"x": 546.3928160444136,
                                            "y": 17.958102836071195},
                                        {"x": 547.7251977470074,
                                            "y": 16.268694580160915},
                                    ]
                                ]
                            ]
                        },
                        "meta": {
                            "alpha": 1,
                            "color": "#FE9573",
                            "visible": True,
                            "z_index": 2,
                        },
                        "multiple": True,
                    },
                    "num": 2,
                    "properties": [],
                },
            ],
            "id": "1af98de3-d356-410a-86c8-3a7b71411575",
            "properties": [],
            "tracking_id": 2,
        },
        {
            "annotation_type": "keypoint",
            "class_id": "f1e62c91-7241-4d42-8e6c-5a4f79f09b3f",
            "class_name": "Person",
            "frames": [
                {
                    "annotation": {
                        "coord": {
                            "points": [
                                {
                                    "name": "nose tip",
                                    "state": {"valid": False, "visible": False},
                                    "x": None,
                                    "y": None,
                                },
                                {
                                    "name": "mouth left corner",
                                    "state": {"valid": False, "visible": False},
                                    "x": None,
                                    "y": None,
                                },
                            ]
                        },
                        "meta": {
                            "alpha": 1,
                            "color": "#6648FF",
                            "visible": True,
                            "z_index": 3,
                        },
                    },
                    "num": 0,
                    "properties": [],
                },
                {
                    "annotation": {
                        "coord": {
                            "points": [
                                {
                                    "name": "nose tip",
                                    "state": {"valid": False, "visible": False},
                                    "x": None,
                                    "y": None,
                                },
                                {
                                    "name": "mouth left corner",
                                    "state": {"valid": False, "visible": False},
                                    "x": None,
                                    "y": None,
                                },
                            ]
                        },
                        "meta": {
                            "alpha": 1,
                            "color": "#6648FF",
                            "visible": True,
                            "z_index": 3,
                        },
                    },
                    "num": 2,
                    "properties": [],
                },
            ],
            "id": "a452943e-bb5e-4c40-8d34-0e322d7f4bf6",
            "properties": [
                {
                    "option_id": "cfd5be2e-695f-468d-8e98-182769728920",
                    "option_name": "Client",
                    "property_id": "ad7647bd-f2a9-4ab4-9ca1-5b7600acfd78",
                    "property_name": "Person Subclass",
                    "type": "radio",
                }
            ],
            "tracking_id": 3,
        },
        {
            "annotation_type": "box",
            "class_id": "88164746-1f8b-4730-8aec-ebd089cbe9d6",
            "class_name": "Penguin",
            "frames": [
                {
                    "annotation": {
                        "coord": {
                            "height": 52.03273942246172,
                            "width": 32.07547252186159,
                            "x": 39.730742505428886,
                            "y": 52.91824642277193,
                        },
                        "meta": {
                            "alpha": 1,
                            "color": "#39F8F8",
                            "visible": True,
                            "z_index": 4,
                        },
                    },
                    "num": 0,
                    "properties": [
                        {
                            "option_ids": ["1", "2", "3"],
                            "option_names": ["Sad", "Happy", "Thoughtful"],
                            "property_id": "ac3bc093-810a-4491-9fc9-3592213aaf3a",
                            "property_name": "Mood(PF)",
                            "type": "checkbox",
                        },
                        {
                            "property_id": "82f0f333-5247-4d3f-bd42-d03b8e914f10",
                            "property_name": "Is area crowded",
                            "type": "free response",
                            "value": "",
                        },
                        {
                            "property_id": "93c6434d-af9c-42b4-999c-911c43b72bb6",
                            "property_name": "Is area cluttered",
                            "type": "free response",
                            "value": "",
                        },
                    ],
                },
                {
                    "annotation": {
                        "coord": {
                            "height": 52.03273942246172,
                            "width": 32.07547252186159,
                            "x": 39.730742505428886,
                            "y": 52.91824642277193,
                        },
                        "meta": {
                            "alpha": 1,
                            "color": "#39F8F8",
                            "visible": True,
                            "z_index": 4,
                        },
                    },
                    "num": 1,
                    "properties": [
                        {
                            "option_ids": ["1"],
                            "option_names": ["Sad"],
                            "property_id": "ac3bc093-810a-4491-9fc9-3592213aaf3a",
                            "property_name": "Mood(PF)",
                            "type": "checkbox",
                        },
                        {
                            "property_id": "82f0f333-5247-4d3f-bd42-d03b8e914f10",
                            "property_name": "Is area crowded",
                            "type": "free response",
                            "value": "",
                        },
                        {
                            "property_id": "93c6434d-af9c-42b4-999c-911c43b72bb6",
                            "property_name": "Is area cluttered",
                            "type": "free response",
                            "value": "",
                        },
                    ],
                },
                {
                    "annotation": {
                        "coord": {
                            "height": 52.03273942246172,
                            "width": 32.07547252186159,
                            "x": 39.730742505428886,
                            "y": 52.91824642277193,
                        },
                        "meta": {
                            "alpha": 1,
                            "color": "#39F8F8",
                            "visible": True,
                            "z_index": 4,
                        },
                    },
                    "num": 2,
                    "properties": [
                        {
                            "option_ids": ["1", "2", "3"],
                            "option_names": ["Sad", "Happy", "Thoughtful"],
                            "property_id": "ac3bc093-810a-4491-9fc9-3592213aaf3a",
                            "property_name": "Mood(PF)",
                            "type": "checkbox",
                        },
                        {
                            "property_id": "82f0f333-5247-4d3f-bd42-d03b8e914f10",
                            "property_name": "Is area crowded",
                            "type": "free response",
                            "value": "yes",
                        },
                        {
                            "property_id": "93c6434d-af9c-42b4-999c-911c43b72bb6",
                            "property_name": "Is area cluttered",
                            "type": "free response",
                            "value": "0",
                        },
                        {
                            "option_id": "1",
                            "option_name": "T",
                            "property_id": "fc5cb926-8b6a-443f-8d67-426478f4056f",
                            "property_name": "Truncated(PF)",
                            "type": "radio",
                        },
                    ],
                },
            ],
            "id": "a518d610-cd99-4975-ae17-6c1a081aeeb1",
            "properties": [
                {
                    "option_ids": ["3"],
                    "option_names": ["Sad"],
                    "property_id": "57d8115c-7a44-45ae-98c6-eb4b2fb9b710",
                    "property_name": "Mood",
                    "type": "checkbox",
                },
                {
                    "property_id": "d099a83f-12e0-423c-a9be-4f1dbd221690",
                    "property_name": "Describe mood",
                    "type": "free response",
                    "value": "",
                },
                {
                    "option_id": "1",
                    "option_name": "Yes",
                    "property_id": "38a1be62-42c2-4c55-b793-136b8c147615",
                    "property_name": "Truncated",
                    "type": "radio",
                },
                {
                    "option_ids": ["3"],
                    "option_names": ["S"],
                    "property_id": "df27390a-4f0c-40c7-9f0f-bacf999cdaee",
                    "property_name": "Size",
                    "type": "checkbox",
                },
                {
                    "property_id": "06bb4da1-d8fd-475b-ad2f-4d4b8b94fb8d",
                    "property_name": "Describe colors",
                    "type": "free response",
                    "value": "1",
                },
                {
                    "option_id": "1",
                    "option_name": "True",
                    "property_id": "c58fe3d6-1693-4995-8e8f-7929b484da1b",
                    "property_name": "Occluded",
                    "type": "radio",
                },
            ],
            "tracking_id": 4,
        },
    ]



import random
from uuid import uuid4
from typing import Optional, List, Union

from ..common.label import ClassType
from .utils import is_sublist
from .deprecated import deprecated


class ObjectInterfaceDef:
    def __init__(
        self,
        keypoints: list = [],
        object_groups: list = [],
        object_classes: list = [],
        annotation_types: list = [],
    ):
        self.keypoints = keypoints
        self.object_groups = object_groups
        self.object_classes = object_classes
        self.annotation_types = annotation_types

    @classmethod
    def get_default(cls):
        return cls(
            keypoints=[],
            object_groups=[],
            object_classes=[],
            annotation_types=[],
        )

    @classmethod
    def from_dict(cls, object_detection_dict: dict):
        return cls(
            keypoints=object_detection_dict["keypoints"],
            object_groups=object_detection_dict["object_groups"],
            object_classes=object_detection_dict["object_classes"],
            annotation_types=object_detection_dict["annotation_types"],
        )

    @deprecated("Use [set_object_group] instead")
    def add_object_group(self, name: str, groups: list, id: str = None):
        # TODO: How to use this method? MinjuneL
        id = str(uuid4()) if id is None else id
        new_group = {"id": id, "name": name, "object_class_ids": []}
        for object in self.object_classes:
            if object["name"] in groups:
                new_group["object_class_ids"].append(object["id"])
                groups.remove(object["name"])
        if len(groups) > 0:
            return print(f"[ERROR] {groups} does not exist")
        self.object_groups.append(new_group)

    def set_object_group(
        self,
        name: Optional[str] = None,
        id: Optional[str] = None,
        groups: List[dict] = []
    ):
        # Validation
        for group in groups:
            if not is_sublist(
                ["id", "name", "annotation_type"],
                group.keys(),
            ):
                raise Exception(
                    "Invalid group format. {`id`, `name`, `annotation_type`} are required.")

        id = str(uuid4()) if id is None else id
        index = None
        for index, item in enumerate(self.object_groups):
            if name is not None and item["name"] == name:
                index = index
                break
            elif item["id"] == id:
                index = index
                break
        if name is None and index is None:
            raise Exception("Provide name if you want to make new group.")

        # Build new group
        new_group = {"id": id, "name": name, "object_class_ids": [
            group["id"] for group in groups]}

        # Set group
        if index is None:
            self.object_groups.append(new_group)
        else:
            self.object_groups[index] = new_group

    def get_object_class(
        self,
        name: str,
        annotation_type: Union[ClassType, str],
    ) -> Optional[dict]:
        if isinstance(annotation_type, ClassType):
            annotation_type = annotation_type.value
        for object in self.object_classes:
            if (
                object["name"] == name and
                object["annotation_type"] == annotation_type
            ):
                return object
        return None

    def add_object_class(
        self,
        name: str,
        annotation_type: str,
        color: str = None,
        properties: list = [],
        id: str = None,
    ):
        for object in self.object_classes:
            if (
                object["name"] == name and
                object["annotation_type"] == annotation_type
            ):
                return print(f"[ERROR] {name} of {annotation_type} already exists")
        id = str(uuid4()) if id is None else id

        def r():
            return random.randint(0, 255)

        self.object_classes.append(
            {
                "id": id,
                "name": name,
                "color": color if color else "#%02X%02X%02X" % (r(), r(), r()),
                "properties": properties,
                "constraints": {},
                "ai_class_map": [],
                "annotation_type": annotation_type,
            }
        )
        if not annotation_type in self.annotation_types:
            self.annotation_types.append(annotation_type)

    def add_box(
        self,
        name: str,
        id: str = None,
        color: str = None,
        properties: list = [],
    ):
        self.add_object_class(name, "box", color, properties, id)

    def add_rbox(
        self,
        name: str,
        id: str = None,
        color: str = None,
        properties: list = [],
    ):
        self.add_object_class(name, "rbox", color, properties, id)

    def add_polyline(
        self,
        name: str,
        id: str = None,
        color: str = None,
        properties: list = [],
    ):
        self.add_object_class(name, "polyline", color, properties, id)

    def add_polygon(
        self,
        name: str,
        id: str = None,
        color: str = None,
        properties: list = [],
    ):
        self.add_object_class(name, "polygon", color, properties, id)

    def add_2dcuboid(
        self,
        name: str,
        id: str = None,
        color: str = None,
        properties: list = [],
    ):
        self.add_object_class(name, "cuboid2d", color, properties, id)

    def add_keypoint(
        self,
        name: str,
        keypoint_id: str,
        keypoint_template: dict = None,
        color: str = None,
        properties: list = [],
        id: str = None,
    ):
        for object in self.object_classes:
            if (
                object["name"] == name and
                object["annotation_type"] == "keypoint"
            ):
                return print(f"[ERROR] {name} of keypoint already exists")
        template_exists = False
        if keypoint_template is None:
            current_keypoints = self.keypoints
            if len(current_keypoints) == 0:
                return print(
                    f"[ERROR] {keypoint_id} does not exist please add a keypoint template"
                )
            else:
                for template in current_keypoints:
                    if template["id"] == keypoint_id:
                        template_exists = True
                if not template_exists:
                    return print(
                        f"[ERROR] {keypoint_id} does not exist please add a keypoint template"
                    )

        id = str(uuid4()) if id is None else id

        def r():
            return random.randint(0, 255)

        self.object_classes.append(
            {
                "id": id,
                "name": name,
                "color": color if color else "#%02X%02X%02X" % (r(), r(), r()),
                "properties": properties,
                "constraints": {},
                "ai_class_map": [],
                "annotation_type": "keypoint",
                "keypoint_interface_id": keypoint_id,
            }
        )
        if not template_exists:
            keypoint_template["id"] = keypoint_id
            self.keypoints.append(keypoint_template)
        if not "keypoint" in self.annotation_types:
            self.annotation_types.append("keypoint")

    def add_object_property(
        self,
        property: dict,
        name: Optional[str] = None,
        id: Optional[str] = None,
        annotation_type: Optional[str] = None
    ):
        object = None
        if id is not None:
            for object in self.object_classes:
                if object["id"] == id:
                    break
        elif name is not None and annotation_type is not None:
            for object in self.object_classes:
                if object["name"] == name and object["annotation_type"] == annotation_type:
                    break
        else:
            return print("[ERROR] Please provide either id or name and annotation_type")
        if object is None:
            return print("[ERROR] Object not found")
        object["properties"].append(property)

    def remove_objects_by_class_id(self, id: str):
        annotation_type_list = []
        keypoint_id_list = []
        for object in self.object_classes:
            if object["id"] == id:
                annotation_type = object["annotation_type"]
                self.object_classes.remove(object)
                if annotation_type == "keypoint":
                    keypoint_id = object["keypoint_interface_id"]
            else:
                annotation_type = object["annotation_type"]
                annotation_type_list.append(annotation_type)
                if annotation_type == "keypoint":
                    keypoint_id_list.append(object["keypoint_interface_id"])

        if annotation_type == "keypoint":
            for keypoint in self.keypoints:
                if keypoint_id not in keypoint_id_list:
                    self.keypoints.remove(keypoint)
                    break

        if not annotation_type in annotation_type_list:
            self.result["annotation_types"].remove(annotation_type)

    def remove_objects_by_class_name(self, name: str, annotation_type: str):
        for object in self.object_classes:
            if (
                object["name"] == name and
                object["annotation_type"] == annotation_type
            ):
                self.remove_objects_by_class_id(id=object["id"])
                return

    def to_dict(self):
        return {
            "keypoints": self.keypoints,
            "object_groups": self.object_groups,
            "object_classes": self.object_classes,
            "annotation_types": list(self.annotation_types),
        }



from .. import __version__
from .label_interface import LabelInterface


class ProjectInfo:
    @staticmethod
    def get_default_w_label_interface(
        name: str,
        label_interface: LabelInterface,
        allow_advanced_qa: bool = False,
        is_public: bool = False,
    ):
        return {
            "name": name,
            "workapp": "image-siesta",
            "label_interface": label_interface.to_dict(),
            "settings": {"allow_advanced_qa": allow_advanced_qa},
            "is_public": is_public,
        }

    @staticmethod
    def get_default(name: str):
        return ProjectInfo.get_default_w_label_interface(
            name=name, label_interface=LabelInterface.get_default()
        )



from uuid import uuid4

from . import __version__
from ..label_interface import LabelInterface


class LabelInfo:
    @classmethod
    def _set_properties(cls, properties_def, properties):
        prop_def_map = {
            prop_def["name"]: prop_def for prop_def in properties_def
        }
        converted_properties = []
        for prop in properties:
            prop_def = prop_def_map[prop["name"]]

            if prop_def["type"] in ["radio", "dropdown", "checkbox"]:
                opt_map = {opt["name"]: opt for opt in prop_def["options"]}

                if prop_def["type"] == "checkbox":
                    option_id = [opt_map[val]["id"] for val in prop["value"]]
                    option_name = [
                        opt_map[val]["name"] for val in prop["value"]
                    ]
                else:
                    option_id = opt_map[prop["value"]]["id"]
                    option_name = opt_map[prop["value"]]["name"]

                converted_properties.append(
                    {
                        "propertyId": prop_def["id"],
                        "propertyName": prop_def["name"],
                        "optionId": option_id,
                        "optionName": option_name,
                    }
                )
            elif prop_def["type"] == "free response":
                converted_properties.append(
                    {
                        "propertyId": prop_def["id"],
                        "propertyName": prop_def["name"],
                        "value": prop["value"],
                    }
                )
        return converted_properties

    @classmethod
    def _get_properties(cls, properties_def, properties):
        prop_def_map = {
            prop_def["name"]: prop_def for prop_def in properties_def
        }
        converted_properties = []
        for prop in properties:
            prop_def = prop_def_map[prop["propertyName"]]

            if prop_def["type"] in ["radio", "dropdown", "checkbox"]:
                converted_properties.append(
                    {"name": prop["propertyName"], "value": prop["optionName"]}
                )
            elif prop_def["type"] == "free response":
                converted_properties.append(
                    {
                        "name": prop["propertyName"],
                        "value": prop["value"],
                    }
                )
        return converted_properties

    def __init__(self, label_interface, result=None):
        if type(label_interface) is dict:
            label_interface = LabelInterface.from_dict(label_interface)
        if result is None:
            result = {
                "categories": {
                    "frames": [],
                    "properties": [],
                },
                "objects": [],
            }
        self.label_interface = label_interface
        self.result = result
        self.object_classes_map = {
            object_class["name"]: object_class
            for object_class in label_interface.object_tracking.object_classes
        }

    def init_objects(self):
        self.result["objects"] = []

    def init_categories(self):
        self.result["categories"] = {"frames": [], "properties": []}

    def add_object(
        self, tracking_id, class_name, annotations, properties=None, id=None
    ):
        id = str(uuid4()) if id is None else id

        self.result["objects"].append(
            {
                "id": id,
                "trackingId": tracking_id,
                "classId": self.object_classes_map[class_name]["id"],
                "className": class_name,
                "annotationType": self.object_classes_map[class_name][
                    "annotation_type"
                ],
                "frames": [
                    {
                        "num": anno["frame_num"],
                        "annotation": {
                            "multiple": anno.get("multiple", False),
                            "coord": anno["coord"],
                            "meta": anno.get("meta", {}),
                        },
                        "properties": LabelInfo._set_properties(
                            self.object_classes_map[class_name]["properties"],
                            anno.get("properties", []),
                        ),
                    }
                    for anno in annotations
                ],
                "properties": LabelInfo._set_properties(
                    self.object_classes_map[class_name]["properties"],
                    properties if properties is not None else [],
                ),
            }
        )

    def get_objects(self):
        try:
            simple_objects = [
                {
                    "id": obj["id"],
                    "tracking_id": obj["trackingId"],
                    "class_name": obj["className"],
                    "annotations": [
                        {
                            "frame_num": frame["num"],
                            "multiple": frame["annotation"].get(
                                "multiple", False
                            ),
                            "coord": frame["annotation"]["coord"],
                            "properties": LabelInfo._get_properties(
                                self.object_classes_map[obj["className"]][
                                    "properties"
                                ],
                                frame["properties"],
                            ),
                        }
                        for frame in obj["frames"]
                    ],
                    "properties": LabelInfo._get_properties(
                        self.object_classes_map[obj["className"]][
                            "properties"
                        ],
                        obj["properties"],
                    ),
                }
                for obj in self.result["objects"]
            ]
            return simple_objects
        except Exception as e:
            return []

    def set_categories(self, frames=None, properties=None):
        self.result["categories"] = {
            "frames": [
                {
                    "num": frame["num"],
                    "properties": LabelInfo._set_properties(
                        self.label_interface.categorization.properties,
                        frame["properties"],
                    ),
                }
                for frame in (frames if frames is not None else [])
            ],
            "properties": LabelInfo._set_properties(
                self.label_interface.categorization.properties,
                properties if properties is not None else [],
            ),
        }

    def get_categories(self):
        try:
            simple_categories = {
                "frames": [
                    {
                        "num": frame["num"],
                        "properties": LabelInfo._get_properties(
                            self.label_interface.categorization.properties,
                            frame["properties"],
                        ),
                    }
                    for frame in self.result["categories"]["frames"]
                ],
                "properties": LabelInfo._get_properties(
                    self.label_interface.categorization.properties,
                    self.result["categories"]["properties"],
                ),
            }
            return simple_categories
        except:
            return {
                "frames": [],
                "properties": [],
            }

    def build_tags(self):
        classes_count = {}
        anno_count = {}
        classes_name = {}
        for obj in self.result["objects"]:
            classes_count[obj["classId"]] = (
                classes_count.get(obj["classId"], 0) + 1
            )
            anno_count[obj["classId"]] = anno_count.get(
                obj["classId"], 0
            ) + len(obj["frames"])
            classes_name[obj["classId"]] = obj["className"]

        return {
            "classes_id": list(classes_count.keys()),
            "class": list(classes_name.values()),
            "classes_count": [
                {
                    "id": k,
                    "name": classes_name[k],
                    "count": v,
                }
                for k, v in classes_count.items()
            ],
            "classes_annotation_count": [
                {
                    "id": k,
                    "name": classes_name[k],
                    "count": v,
                }
                for k, v in anno_count.items()
            ],
        }

    def build_info(self):
        return {
            "version": __version__,
            "meta": {
                "imageInfo": {},
                "editInfo": {
                    "objects": [
                        {
                            "id": obj["id"],
                            "color": self.object_classes_map[obj["className"]][
                                "color"
                            ],
                            "visible": True,
                            "selected": False,
                            "trackingId": obj["trackingId"],
                        }
                        for obj in self.result["objects"]
                    ]
                },
            },
            "result": self.result,
            "tags": self.build_tags(),
        }



import semver

from .label import ClassType


def is_camel_version(label_interface):
    return (
        semver.VersionInfo.parse(label_interface.version).compare("0.4.0") < 0
    )


# __all__ = (
#     "CategorizationDef",
#     "PropertyDef",
#     "PropertyOptionsDef",
#     "PropertyOptionsItemDef",
# )

__all__ = ("ClassType",)



from typing import Optional
from uuid import uuid4

from ..common.label_creator import LabelCreator
from ..common.utils import set_properties, unique_string_builder, is_sublist
from .label import Cuboid
from ..common.label import ClassType


class CuboidCreator(LabelCreator):
    def __init__(self, object_classes_map):
        super().__init__(object_classes_map)

    def get_default_cuboid_coord(self):
        return {
            "position": {
                "x": 0.0,
                "y": 0.0,
                "z": 0.0
            },
            "rotation_quaternion": {
                "x": 0,
                "y": 0,
                "z": 0,
                "w": 0
            },
            "size": {
                "x": 0,
                "y": 0,
                "z": 0
            }
        }

    def create(
        self,
        class_name: str,
        frames: list,
        meta: dict = {},
        properties: list = [],
        id: str = str(uuid4()),
        tracking_id: Optional[int] = None,
    ):
        assert unique_string_builder(
            class_name,
            ClassType.CUBOID,
        ) in self.object_classes_map.keys()
        coord = frames[0]["annotation"]["coord"]
        assert is_sublist(
            ["position", "rotation_quaternion", "size"],
            coord.keys(),
        )
        return Cuboid(
            id=id,
            class_name=class_name,
            class_id=self.object_classes_map[
                unique_string_builder(
                    class_name,
                    ClassType.CUBOID,
                )
            ]["id"],
            frames=frames,
            meta=meta if meta else self._get_meta(
                class_name, ClassType.CUBOID),
            properties_def=self.object_classes_map[
                unique_string_builder(
                    class_name,
                    ClassType.CUBOID,
                )
            ]["properties"],
            properties=properties,
            tracking_id=tracking_id,
        )
